<?php

namespace Service;

class MultiOrder {
    
    protected $util;
    protected $orderController;
    protected $orderService;
    protected $connection;
    
    public function __construct(\Util\Util $util,
                                \Controller\Order $orderController,
                                \Service\Order $orderService,
                                $connection){
        $this->util = $util;
        $this->orderController = $orderController;
        $this->orderService = $orderService;
        $this->connection = $connection;
    }
    
    public function separateItems($items){
        $realItemsQuant = 0;
        for($i = 0; $i < count($items); $i++)
            $realItemsQuant += $this->orderController->divideItems($items, $items[$i]);
        
        unset($item);
        return array(
            'itemsQuant' => $realItemsQuant,
            'minOrderValue' => $this->util->div($realItemsQuant, 100),
            'items' => $items
        ); 
    }
    
    private function getTotalOrderDiscount($cdfilial, $nrcomanda){
        $params = array(
            'NRCOMANDA' => $nrcomanda,
            'CDFILIAL' => $cdfilial
        );
		return $this->connection->fetchAssoc(\Util\Query::GET_ORDER_DISCOUNT, $params)['DISCOUNT'];
    }
    
    public function shareDiscount($items, $totalDiscount, $orderDiscount, $productsValue, $remaingValue, $remaingDiscount){
        while($orderDiscount > 0){
            $sumValue = 0;
            
            foreach ($items as &$item)
               $this->orderController->shareDiscount($item, $orderDiscount, $productsValue, $remaingValue, $remaingDiscount, false);
            
            unset($item);
            $remaingValue = $productsValue;
            foreach ($items as $item)
                $sumValue += $this->orderController->getDiscountValue($item);
            
            
            $orderDiscount = $this->util->subtract($totalDiscount, $sumValue);
            $remaingDiscount = $orderDiscount;
            unset($item);
        }
        return $items;
    }
    
    public function validateOrder($minOrderValue, $productsValue, $orderDiscount, $totalValue){
        if($this->util->subtract($productsValue, $orderDiscount) != $totalValue)
            throw new \Exception("totalValue inconsistente!");
        if($productsValue <= $orderDiscount) 
            throw new \Exception("Desconto maior que valor dos itens!");
        if($totalValue < $minOrderValue) 
            throw new \Exception("Valor total inválido menor que valor mínimo dos itens");
    }
    
    public function validateConsumer($cdcliente, $cdconsumidor){
        if((!empty($cdcliente) && !empty($cdconsumidor)) && $this->util->consumerExists($cdcliente, $cdconsumidor) === false)
            throw new \Exception('Consumidor inexistente.');
    }
    
    public function insertComandaVen($cdfilial, $nrvendarest, $nrcomanda, $cdloja, $idstcomanda, $dscomanda, $idorgcmdvenda, $idretballoja, $nrcomandaext, $idimpcupexp, $dsconsumidor, $closeByApi){
        $sgsexocon = 'M';
        $txmotivcance = ' ';
        $cdpais = '0055';
        if($idorgcmdvenda == "MES_MOB" || $idorgcmdvenda == "CMD_MOB"){
            $dscomanda = "PKR_$nrcomanda";
            $toGo = 'N';
        }else{
            $dscomanda = "DLV_$nrcomanda";
        }
        $params = array(
            'CDFILIAL'        => $cdfilial,
            'NRVENDAREST'     => $nrvendarest,
            'NRCOMANDA'       => $nrcomanda,
            'CDLOJA'          => $cdloja,
            'IDSTCOMANDA'     => $idstcomanda,
            'DSCOMANDA'       => $dscomanda,
            'SGSEXOCON'       => $sgsexocon,
            'TXMOTIVCANCE'    => $txmotivcance,
            'VRACRCOMANDA'    => 0,
            'CDPAIS'          => $cdpais,
            'SGESTADO'        => null,
            'CDMUNICIPIO'     => null,
            'NRCEPCONSCOMAND' => null,
            'CDBAIRRO'        => null,
            'DSBAIRRO'        => null,
            'DSENDECONSCOMAN' => null,
            'DSCOMPLENDCOCOM' => null,
            'DSREFENDCONSCOM' => null,
            'IDORGCMDVENDA'   => $idorgcmdvenda,
            'IDRETBALLOJA'    => $idretballoja,
            'NRCOMANDAEXT'    => $nrcomandaext,
            'IDSINCAGENDA'    => 'N',
            'DTHRAGENDADA'    => null,
            'IDIMPCUPEXP'     => $idimpcupexp,
            'DSCONSUMIDOR'    => $dsconsumidor,
            'VRDESCFID'       => 0,
            'DSOBSCOMANDA'    => null,
            'IDFECHAMENTOAPI' => $closeByApi,
            'NRCPFFIDUDS'     => null,
            'VRDESCCASHUDS'   => 0,
        );
        $comandavenStatus = $this->orderService->insertComandaven($params);
        if($comandavenStatus['error'] === true)
            throw new \Exception($comandavenStatus['message']);
        return $comandavenStatus;
    }
    
    public function setIdimpcupexp($idorgcmdvenda, $idretballoja, $printMode){
        $idimpcupexp = '0';
                    
        $taaOrder = strpos($idorgcmdvenda,'TAA') !== false;
        if($printMode == 'EXPEDICAO'){
            if($idretballoja == 'S' || $taaOrder) $idimpcupexp = '1';
        }else if($printMode == 'INICIO'){
            if($taaOrder) $idimpcupexp = '2';
            else if($idretballoja == 'S') $idimpcupexp = '1';
        }
        
        return $idimpcupexp;
    }
    
    public function saveItems($items, $cdfilial, $nrvendarest, $nrcomanda, $idcontroprod, $cdcaixa, $cdloja, $cdvendedor, $nrinscrcons, $cdsenhaped, $idorgcmdvenda, $nrprodcomven, $driveThruData){
        $positionNumber = '01';
        $itemNum = 0;
        $toTravelData = array();
        $holdTime = 0;
        $nrseqprodcup = $this->orderService->generateItemsSeq($cdfilial);
        foreach($items as &$item)
            $this->orderController->saveItem($item, $cdfilial, $nrvendarest, $nrcomanda, $nrseqprodcup, $positionNumber, $idcontroprod, $cdcaixa, $cdloja, $cdvendedor, $nrinscrcons, $itemNum, null, $toTravelData, $cdsenhaped, $holdTime, $driveThruData, $idorgcmdvenda, false, $nrprodcomven);
        unset($item); 
    }
    
    public function getOrdersValue($cdfilial, $stores){
        
        $orders = array();
        foreach($stores as $order){
            $nrcomada = $order['saleCode'];
            array_push($orders, $nrcomada);
        }
        $params = array(
            'NRCOMANDA' => $orders,
            'CDFILIAL' => $cdfilial
        );
        $paramsType = array(
            'NRCOMANDA' => \Doctrine\DBAL\Connection::PARAM_STR_ARRAY,
            'CDFILIAL' => \PDO::PARAM_STR
        );
		return $this->connection->fetchAssoc(\Util\Query::GET_ORDERS_TOTAL, $params, $paramsType)['TOTAL'];
    }
    
    public function mergeGiftCards($giftCards, $cdfilial){
        $totalValue = 0;
        foreach ($giftCards as $giftCard) 
            $totalValue = bcadd($totalValue, $giftCard['value'],2);
            
        $destinyCard = array_shift($giftCards);
        $extratoCons = new \SimpleXMLElement('<ExtratoCons/>');
        $xmlFields = $this->orderController->getXMLFields();
        foreach ($giftCards as $giftCard) {
            $registro = $extratoCons->addChild('Registro');
            foreach ($xmlFields as $field) 
                $registro->addChild($field);
            $registro->cdcliente    = $giftCard['clientCode'];
            $registro->cdconsumidor = $giftCard['consumerCode'];
            $registro->cdfamilisald = $giftCard['cdfamilisald'];
            $registro->dtmovextcons = date('d/m/Y H:i:s');
            $registro->nrseqmovext  = '001';
            $registro->cdtiporece   = '001';
            $registro->cdfilial     = $cdfilial;
            $registro->cdcaixa      = null;
            $registro->dsopeextcons = "Transf. Saldo:".$destinyCard['cardNumber']." - ". $destinyCard['clientCode'];
            $registro->vrmovextcons = $giftCard['value'];
            $registro->vrsaldconext = 0;
            $registro->idtpmovext   = 'D';
            $registro->idimpextrato = 'S';
            $detiny = $extratoCons->addChild('Registro');
            foreach ($xmlFields as $field) 
                $detiny->addChild($field);
            $detiny->cdcliente    = $destinyCard['clientCode'];
            $detiny->cdconsumidor = $destinyCard['consumerCode'];
            $detiny->cdfamilisald = $destinyCard['cdfamilisald'];
            $detiny->dtmovextcons = date('d/m/Y H:i:s');
            $detiny->nrseqmovext  = '001';
            $detiny->cdtiporece   = '001';
            $detiny->cdfilial     = $cdfilial;
            $detiny->cdcaixa      = null;
            $detiny->dsopeextcons = "Transf. Saldo:".$giftCard['cardNumber']." - ". $giftCard['clientCode'];
            $detiny->vrmovextcons = $giftCard['value'];
            $detiny->vrsaldconext = 0;
            $detiny->idtpmovext   = 'C';
            $detiny->idimpextrato = 'S';
            sleep(1);
        }
        $xmlToSend = $extratoCons->asXML();
        $params = array(
            'P_XMLEXTCONS' => $xmlToSend    
        );  
        $this->util->callExtratocons($params, $cdfilial, "InsereDados");
        
        $destinyCard['value'] = $totalValue;
        return $destinyCard;
    }
    
    public function saveGiftCard($giftCard, $cdfilial, $nrvendarest, $cdcaixa, $dtabercaix, $paymentValue){
        $movementType = 'E';
        $cdcliente    = $giftCard['clientCode'];
        $cdconsumidor = $giftCard['consumerCode'];
        $cdtiporece        = $this->util->getCreditPayment();
        $movcaixadlvSeq    = $this->orderService->generateMovcaixadlvSeq($cdfilial, $nrvendarest);
        $returnMovcaixadlv = $this->orderController->insertMovCaixaDlv($cdfilial, $nrvendarest, $movcaixadlvSeq, $movementType, $cdcliente, $cdconsumidor, $cdtiporece, $paymentValue, null, 1);
        if($returnMovcaixadlv['error'])
            throw new \Exception($returnMovcaixadlv['message']);
    }
    
    public function validateTotalPaid($paymentsValue, $totalGiftCard, $cashBack, $ordersValue){
        $total = $paymentsValue + $totalGiftCard + $cashBack;
        if($total < $ordersValue)
            throw new \Exception("Soma dos pagamentos menor que soma dos items! Pagamentos: $total. Total Pedido: $ordersValue");
    }
    
    public function totalGiftCard($giftCards){
        $giftCardValue = 0;
        foreach($giftCards as $giftCard)
            $giftCardValue = $this->util->add($giftCardValue, $giftCard['value']);
        return $giftCardValue;
    }
    
    public function totalPayments($payments){
        $paymentsValue = 0;
        foreach($payments as $payment)
            $paymentsValue = $this->util->add($paymentsValue, $payment['value']);
        return $paymentsValue;
    }
    
    public function treatPayments($payments, $remaingValue){
        $orderPayments = array();
        foreach($payments as &$payment){
            if($payment['value'] > 0 && $remaingValue > 0){
                $orderPayment = $payment;
                if($payment['value'] >= $remaingValue){
                    $orderPayment['value'] = $remaingValue;
                    array_push($orderPayments, $orderPayment);
                    $payment['value'] = $this->util->subtract($payment['value'], $remaingValue);
                    $remaingValue = 0;
                }else{
                    array_push($orderPayments, $orderPayment);
                    $remaingValue = $this->util->subtract($remaingValue, $payment['value']);
                    $payment['value'] = 0;
                }
            }
        }
        unset($payment);
        $payments = $this->removeEmptyPayments($payments);
        
        return array(
            'payments'      => $payments,
            'remaingValue'  => $remaingValue,
            'orderPayments' => $orderPayments
        );
    }
    
    public function treatGiftCard($giftCard, $cdfilial, $remaingValue){
        $paymentValue = 0;
        if($giftCard['value'] >= $remaingValue){
            $paymentValue = $remaingValue;
            $remaingValue = 0;
            $giftCard['value'] = $this->util->subtract($giftCard['value'], $paymentValue);
        }else{
            $paymentValue = $giftCard['value'];
            $remaingValue = $this->util->subtract($remaingValue, $giftCard['value']);
            $giftCard['value'] = 0;
        }
        return array(
            'giftCard'     => $giftCard,
            'remaingValue' => $remaingValue,
            'paymentValue' => $paymentValue
        );
    }
    
    private function removeEmptyPayments($payments){
        return array_filter($payments, function($payment){
        	return $payment['value'] > 0;
        });
    }
    
    public function treatCashBack($itemsValue, $remaingValue, $cdfilial, $nrcomanda, $nrvendarest, $cdcliente, $cdconsumidor, $cashBack){
        $items = $this->orderService->getOrderItems($nrvendarest, $nrcomanda, $cdfilial);
        $realItemsQuant = count($items);
        $realItemsQuant = $this->util->div($realItemsQuant, 100);
        $items = $this->formatItemsToDiscount($items);
        
        $usedCashBack = 0;
        
        if($remaingValue < $cashBack){
            $orderDiscount = $remaingValue;
            $cashBack = $this->util->subtract($cashBack, $remaingValue);
            $usedCashBack = $remaingValue;
        }else if($cashBack < $itemsValue){
            $orderDiscount = $cashBack;
            $usedCashBack = $cashBack;
            $cashBack = 0;
        }else{
            $cashToApply = $this->util->subtract($itemsValue, $realItemsQuant);
            $orderDiscount = $cashToApply;
            $userCashBack = $cashBack;
            $cashBack = $this->util->add($cashToApply, $realItemsQuant);
            if($userCashBack < $cashBack){
                $cashBack = $userCashBack;
            }
            $usedCashBack = $cashBack;
            $orderPayments = $this->orderService->getOrderPayments($cdfilial, $nrvendarest);
            if(empty($orderPayments)){
                $cdtiporece = $this->util->getMoney();
                $movcaixadlvSeq = str_pad(1, 10, '0', STR_PAD_LEFT);
                $this->orderController->insertMovCaixaDlv($cdfilial, $nrvendarest, $movcaixadlvSeq, 'E', $cdcliente, $cdconsumidor, $cdtiporece, $realItemsQuant, null, 1);
            }
        }
        
        $totalOrderDiscount = $this->getTotalOrderDiscount($cdfilial, $nrcomanda);
        $remaingDiscount = $orderDiscount;
        $totalOrderDiscount = $totalOrderDiscount + $remaingDiscount;
        $items = $this->shareDiscount($items, $totalOrderDiscount, $orderDiscount, $itemsValue, $itemsValue, $remaingDiscount, false);
        foreach($items as $item){
            $cdproduto = $item['CDPRODUTO'];
            $nrprodcomven = $item['NRPRODCOMVEN'];
            $discount = $item['discount'];
            $this->orderService->updateItemDiscount($nrcomanda, $nrvendarest, $cdfilial, $cdproduto, $nrprodcomven, $discount);
        }
        $this->orderService->saveCashBack($nrcomanda, $nrvendarest, $cdfilial, $usedCashBack);
        return $cashBack;
    }
    
    public function formatItemsToDiscount($items){
        foreach($items as &$item){
            $item['quantity'] = $item['QTPRODCOMVEN'];
            $item['unitValue'] = $item['VRPRECCOMVEN'];
            $item['discount'] = $item['VRDESCCOMVEN'];
        }
        unset($item);
        return $items;
    }
}